/*
 * Copyright 2012, 2017 INFN
 *
 * Licensed under the EUPL, Version 1.2 or – as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 * https://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 */

#ifndef __CHAOSFramework__DataWorker__
#define __CHAOSFramework__DataWorker__

//#include "../dataservice_global.h"

#include <chaos/common/chaos_constants.h>
#include <chaos/common/chaos_types.h>
#include <chaos/common/utility/LockableObject.h>
#include <chaos/common/utility/StartableService.h>

#include <boost/thread.hpp>
#include <boost/atomic.hpp>

#include <queue>

#define DEFAULT_JOB_THREAD 1

namespace chaos {
    namespace metadata_service {
		namespace worker {
			
			typedef struct WorkerJob {
				WorkerJob(){};
				virtual ~WorkerJob(){};
            } WorkerJob;
            typedef ChaosSharedPtr<WorkerJob> WorkerJobPtr;
			
			typedef struct DataWorkerSetting {
                unsigned int        instances;
				unsigned int        thread_number;
                unsigned int        queue_push_timeout;
			} DataWorkerSetting;
			
			class DataWorker:
            public chaos::common::utility::StartableService  {
				//job queue list
                std::queue<WorkerJobPtr> job_queue;
				
				//group thread
				boost::thread_group job_thread_group;
				
				//muthex for condition variable
				ChaosMutex mutex_job;
				ChaosMutex mutex_submit;
                
				//condition for the threads
				ChaosConditionVariable consume_job_condition;
				ChaosConditionVariable push_condition;
                
				bool work;
				
                boost::atomic<uint64_t> job_in_queue;
                
                unsigned int max_element;
			protected:
				void * * thread_cookie;
				void consumeJob(void *cookie);
				WorkerJobPtr getNextOrWait(ChaosUniqueLock& lock);
				
				virtual int executeJob(WorkerJobPtr job_info, void* cookie) = 0;
			public:
				DataWorker();
				virtual ~DataWorker();
				void init(void *init_data);
				void start();
				void stop();
				void deinit();
                void setMaxElement(uint64_t new_max_element);
                virtual int submitJobInfo(WorkerJobPtr job_info, int64_t milliseconds_to_wait = chaos::common::constants::MDSHistoryQueuePushTimeoutinMSec);
			};
			
		}
	}
}

#endif /* defined(__CHAOSFramework__DataWorker__) */
